@@ -24,6 +24,7 @@ $_SUDO apt-get install -y \
     build-essential \
     automake \
     cmake \
+    curl \
     libzmq3-dev \
     liblua5.2-dev \
     libconfig-dev \
@@ -51,6 +52,13 @@ $_SUDO apt-get install -y \
     libwebsockets-dev \
     libgpiod-dev \
     libcurl4-openssl-dev \
-    gawk
+    libpci-dev \
+    gawk \
+    cpio \
+    meson \
+    ninja-build \
+    libzstd-dev \
+    wget \
+    python3
 
 $_SUDO ln -sf /usr/lib/x86_64-linux-gnu/pkgconfig/lua5.2.pc /usr/lib/x86_64-linux-gnu/pkgconfig/lua.pc
