@@ -69,6 +69,14 @@ static pthread_t subprocess_ipc_handler_thread_id;
 static pthread_mutex_t subprocess_msg_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t subprocess_wkup = PTHREAD_COND_INITIALIZER;
 
+struct notify_conn {
+	SIMPLEQ_ENTRY(notify_conn) next;
+	int sockfd;
+};
+
+SIMPLEQ_HEAD(connections, notify_conn);
+static struct connections notify_conns;
+
 static bool is_selection_allowed(const char *software_set, char *running_mode,
 				 struct dict const *acceptedlist)
 {
@@ -116,11 +124,61 @@ static void clean_msg(char *msg, char drop)
 	}
 }
 
+static int write_notify_msg(ipc_message *msg, int sockfd)
+{
+	void *buf;
+	size_t count;
+	ssize_t n;
+	int ret = 0;
+
+	buf = msg;
+	count = sizeof(*msg);
+	while (count > 0) {
+		n = send(sockfd, buf, count, MSG_NOSIGNAL);
+		if (n <= 0) {
+			/*
+			 * We can't use the notify methods for error logging here as it will cause a deadlock.
+			 */
+			if (n == 0) {
+				fprintf(stderr, "Error: A status client is not responding, removing it.\n");
+			} else {
+				fprintf(stderr, "A status client disappeared, removing it: %s\n", strerror(errno));
+			}
+			ret = -1;
+			break;
+		}
+		count -= (size_t)n;
+		buf = (char*)buf + n;
+	}
+	return ret;
+}
+
+/*
+ * This must be called after acquiring the mutex
+ * for the msglock structure
+ */
+static void send_notify_msg(ipc_message *msg)
+{
+	struct notify_conn *conn, *tmp;
+	int ret;
+
+	SIMPLEQ_FOREACH_SAFE(conn, &notify_conns, next, tmp) {
+		ret = write_notify_msg(msg, conn->sockfd);
+		if (ret < 0) {
+			close(conn->sockfd);
+			SIMPLEQ_REMOVE(&notify_conns, conn,
+						   notify_conn, next);
+			free(conn);
+		}
+	}
+}
+
 static void network_notifier(RECOVERY_STATUS status, int error, int level, const char *msg)
 {
 	int len = msg ? strlen(msg) : 0;
 	struct msg_elem *newmsg = (struct msg_elem *)calloc(1, sizeof(*newmsg) + len + 1);
 	struct msg_elem *oldmsg;
+	ipc_message ipcmsg;
 
 	if (!newmsg)
 		return;
@@ -148,6 +206,18 @@ static void network_notifier(RECOVERY_STATUS status, int error, int level, const
 
 
 	SIMPLEQ_INSERT_TAIL(&notifymsgs, newmsg, next);
+
+	ipcmsg.magic = IPC_MAGIC;
+	ipcmsg.type = NOTIFY_STREAM;
+	memset(&ipcmsg.data, 0, sizeof(ipcmsg.data));
+
+	strncpy(ipcmsg.data.notify.msg, newmsg->msg,
+			sizeof(ipcmsg.data.notify.msg) - 1);
+	ipcmsg.data.notify.status = newmsg->status;
+	ipcmsg.data.notify.error = newmsg->error;
+	ipcmsg.data.notify.level = newmsg->level;
+	send_notify_msg(&ipcmsg);
+
 	pthread_mutex_unlock(&msglock);
 }
 
@@ -366,7 +436,8 @@ void *network_thread (void *data)
 	struct sockaddr_un cliaddr;
 	ipc_message msg;
 	int nread;
-	struct msg_elem *notification;
+	struct msg_elem *notification, *tmp;
+	struct notify_conn *conn;
 	int ret;
 	update_state_t value;
 	struct subprocess_msg_elem *subprocess_msg;
@@ -378,6 +449,7 @@ void *network_thread (void *data)
 	}
 
 	SIMPLEQ_INIT(&notifymsgs);
+	SIMPLEQ_INIT(&notify_conns);
 	SIMPLEQ_INIT(&subprocess_messages);
 	register_notifier(network_notifier);
 
@@ -409,8 +481,8 @@ void *network_thread (void *data)
 		nread = read(ctrlconnfd, (void *)&msg, sizeof(msg));
 
 		if (nread != sizeof(msg)) {
-			TRACE("IPC message too short: fragmentation not supported (read %d bytes)",
-				nread);
+			TRACE("IPC message too short: fragmentation not supported (read %d bytes, expected %zu bytes)",
+				nread, sizeof(msg));
 			close(ctrlconnfd);
 			continue;
 		}
@@ -507,6 +579,60 @@ void *network_thread (void *data)
 				}
 				pthread_mutex_unlock(&msglock);
 
+				break;
+			case NOTIFY_STREAM:
+				msg.type = ACK;
+				memset(msg.data.msg, 0, sizeof(msg.data.msg));
+				msg.data.status.current = instp->status;
+				msg.data.status.last_result = instp->last_install;
+				msg.data.status.error = instp->last_error;
+
+				ret = write(ctrlconnfd, &msg, sizeof(msg));
+				msg.type = NOTIFY_STREAM;
+				if (ret < 0) {
+					ERROR("Error write notify ack on socket ctrl");
+					close(ctrlconnfd);
+					break;
+				}
+
+				/* Get first notification from the queue */
+				pthread_mutex_lock(&msglock);
+				notification = SIMPLEQ_FIRST(&notifymsgs);
+
+				/* Send notify history */
+				SIMPLEQ_FOREACH_SAFE(notification, &notifymsgs, next, tmp) {
+					memset(msg.data.msg, 0, sizeof(msg.data.msg));
+
+					strncpy(msg.data.notify.msg, notification->msg,
+							sizeof(msg.data.notify.msg) - 1);
+					msg.data.notify.status = notification->status;
+					msg.data.notify.error = notification->error;
+					msg.data.notify.level = notification->level;
+
+					ret = write_notify_msg(&msg, ctrlconnfd);
+					if (ret < 0) {
+						pthread_mutex_unlock(&msglock);
+						ERROR("Error write notify history on socket ctrl");
+						close(ctrlconnfd);
+						break;
+					}
+				}
+
+				/*
+				 * Save the new connection to send notifications to
+				 */
+				conn = (struct notify_conn *)calloc(1, sizeof(*conn));
+				if (!conn) {
+					pthread_mutex_unlock(&msglock);
+					ERROR("Out of memory, skipping...");
+					close(ctrlconnfd);
+					pthread_mutex_unlock(&stream_mutex);
+					continue;
+				}
+				conn->sockfd = ctrlconnfd;
+				SIMPLEQ_INSERT_TAIL(&notify_conns, conn, next);
+				pthread_mutex_unlock(&msglock);
+
 				break;
 			case SET_AES_KEY:
 #ifndef CONFIG_PKCS11
