@@ -139,21 +139,13 @@ static int switch_mode(char *devreset, int resoffset, char *devprog, int progoff
 		goto freegpios;
 	}
 
-#ifdef CONFIG_UCFW_OLD_LIBGPIOD
-	status = gpiod_line_request_output(linereset, RESET_CONSUMER, false, 0);
-#else
 	status = gpiod_line_request_output(linereset, RESET_CONSUMER, 0);
-#endif
 	if (status) {
 		ret  =-ENODEV;
 		ERROR("Cannot request reset line");
 		goto freegpios;
 	}
-#ifdef CONFIG_UCFW_OLD_LIBGPIOD
-	status = gpiod_line_request_output(lineprog, PROG_CONSUMER, false, mode);
-#else
 	status = gpiod_line_request_output(lineprog, PROG_CONSUMER, mode);
-#endif
 	if (status) {
 		ret  =-ENODEV;
 		ERROR("Cannot request prog line");
@@ -533,7 +525,12 @@ static int install_uc_firmware_image(struct img_type *img,
 
 		gpio = (cnt == 0) ? &hnd_data.reset : &hnd_data.prog;
 
-		get_gpio_from_property(properties, gpio);
+		ret = get_gpio_from_property(properties, gpio) < 0;
+		if (ret < 0) {
+			ERROR("Cannot extract GPIO from properties");
+			return ret;
+		}
+
 		DEBUG("line %s : device %s, num = %d, active_low = %s",
 			properties_list[cnt],
 			gpio->gpiodev,
