@@ -35,7 +35,8 @@ typedef enum {
 	SOURCE_WEBSERVER,
 	SOURCE_SURICATTA,
 	SOURCE_DOWNLOADER,
-	SOURCE_LOCAL
+	SOURCE_LOCAL,
+	SOURCE_CHUNKS_DOWNLOADER
 } sourcetype;
 
 #ifdef __cplusplus
